@@ -2,6 +2,7 @@
  *									    *
  *   BeOS Randomness-Gathering Code					    *
  *   Copyright Peter Gutmann, Paul Kendall, and Chris Wedgwood 1996-1998    *
+ *   Copyright (C) 1998, 1999  Werner Koch
  *									    *
  ****************************************************************************/
 
@@ -560,7 +561,7 @@ start_gatherer( int pipefd )
 	if( s ) {
 	    dbgfp = (*s=='-' && !s[1])? stdout : fopen(s, "a");
 	    if( !dbgfp )
-		g10_log_info("can't open debug file '%s': %s\n",
+		g10_log_info("can't open debug file `%s': %s\n",
 			     s, strerror(errno) );
 	    else
 		fprintf(dbgfp,"\nSTART RNDUNIX DEBUG pid=%d\n", (int)getpid());
@@ -627,7 +628,7 @@ start_gatherer( int pipefd )
 	    while( write( pipefd, &msg, sizeof(msg) ) != sizeof(msg) ) {
 		if( errno == EINTR )
 		    continue;
-		if( errno = EAGAIN ) {
+		if( errno == EAGAIN ) {
 		    struct timeval tv;
 		    tv.tv_sec = 0;
 		    tv.tv_usec = 50000;
@@ -678,6 +679,9 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
     size_t n;
 
     if( !gatherer_pid ) {
+	/* make sure we are not setuid */
+	if( getuid() != geteuid() )
+	    BUG();
 	/* time to start the gatherer process */
 	if( pipe( pipedes ) ) {
 	    g10_log_error("pipe() failed: %s\n", strerror(errno));
@@ -742,7 +746,7 @@ gather_random( void (*add)(const void*, size_t, int), int requester,
 #ifndef IS_MODULE
 static
 #endif
-const char * const gnupgext_version = "RNDUNIX ($Revision: 1.4 $)";
+const char * const gnupgext_version = "RNDUNIX ($Revision: 1.7 $)";
 
 
 static struct {
