@@ -1,5 +1,5 @@
 /* armor.c - Armor flter
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -29,6 +29,7 @@
 #include "iobuf.h"
 #include "memory.h"
 #include "util.h"
+#include "iobuf.h"
 #include "filter.h"
 #include "packet.h"
 #include "options.h"
@@ -37,6 +38,8 @@
 #include "i18n.h"
 
 
+#define MAX_LINELEN 20000
+
 #define CRCINIT 0xB704CE
 #define CRCPOLY 0X864CFB
 #define CRCUPDATE(a,c) do {						    \
@@ -86,6 +89,7 @@ typedef enum {
 
 /* if we encounter this armor string with this index, go
  * into a mode which fakes packets and wait for the next armor */
+#define BEGIN_SIGNATURE 2
 #define BEGIN_SIGNED_MSG_IDX 3
 static char *head_strings[] = {
     "BEGIN PGP MESSAGE",
@@ -109,12 +113,6 @@ static char *tail_strings[] = {
 };
 
 
-static fhdr_state_t find_header( fhdr_state_t state,
-				 byte *buf, size_t *r_buflen,
-				 IOBUF a, size_t n,
-				 unsigned *r_empty, int *r_hashes,
-				 int only_keyblocks, int *not_dashed );
-
 
 static void
 initialize(void)
@@ -153,7 +151,7 @@ initialize(void)
  * Returns: True if it seems to be armored
  */
 static int
-is_armored( byte *buf )
+is_armored( const byte *buf )
 {
     int ctb, pkttype;
 
@@ -256,484 +254,236 @@ parse_hash_header( const char *line )
 }
 
 
-/****************
- * parse an ascii armor.
- * Returns: the state,
- *	    the remaining bytes in BUF are returned in RBUFLEN.
- *	    r_empty return the # of empty lines before the buffer
- */
-static fhdr_state_t
-find_header( fhdr_state_t state, byte *buf, size_t *r_buflen,
-	     IOBUF a, size_t n, unsigned *r_empty, int *r_hashes,
-	     int only_keyblocks, int *not_dashed )
+
+
+static unsigned
+trim_trailing_spaces( byte *line, unsigned len )
 {
-    int c=0, i;
-    const char *s;
-    byte *p;
-    size_t buflen;
-    int cont;
-    int clearsig=0;
-    int hdr_line=0;
-    unsigned empty = 0;
+    byte *p, *mark;
+    unsigned n;
 
-    buflen = *r_buflen;
-    assert(buflen >= 100 );
-    buflen -= 4; /* reserved room for CR,LF, and two extra */
-
-    do {
-	switch( state ) {
-	  case fhdrHASArmor:
-	    /* read at least the first byte to check whether it is armored
-	     * or not */
-	    c = 0;
-	    for(n=0; n < 28 && (c=iobuf_get(a)) != -1 && c != '\n'; )
-		buf[n++] = c;
-	    if( !n && c == '\n' )
-		state = fhdrCHECKBegin;
-	    else if( !n  || c == -1 )
-		state = fhdrNOArmor; /* too short */
-	    else if( !is_armored( buf ) ) {
-		state = fhdrNOArmor;
-		if( c == '\n' )
-		    buf[n++] = c;
-	    }
-	    else if( c == '\n' )
-		state = fhdrCHECKBegin;
-	    else
-		state = fhdrINITCont;
-	    break;
-
-	  case fhdrINIT: /* read some stuff into buffer */
-	    n = 0;
-	  case fhdrINITCont: /* read more stuff into buffer */
-	    c = 0;
-	    for(; n < buflen && (c=iobuf_get(a)) != -1 && c != '\n'; )
-		buf[n++] = c;
-	    state = c == '\n' ? fhdrCHECKBegin :
-		     c == -1  ? fhdrEOF : fhdrINITSkip;
-	    break;
-
-	  case fhdrINITSkip:
-	    if( c == '\n' )
-		n = 0;
-	    else {
-		while( (c=iobuf_get(a)) != -1 && c != '\n' )
-		    ;
-	    }
-	    state =  c == -1? fhdrEOF : fhdrINIT;
-	    break;
-
-	  case fhdrSKIPHeader:
-	    while( (c=iobuf_get(a)) != -1 && c != '\n' )
-		;
-	    state =  c == -1? fhdrEOF : fhdrWAITHeader;
-	    break;
-
-	  case fhdrWAITHeader: /* wait for Header lines */
-	    c = 0;
-	    for(n=0; n < buflen && (c=iobuf_get(a)) != -1 && c != '\n'; )
-		buf[n++] = c;
-	    buf[n] = 0;
-	    if( n < buflen || c == '\n' ) {
-		if( n && buf[0] != '\r') { /* maybe a header */
-		    if( strchr( buf, ':') ) { /* yes */
-			int hashes=0;
-			if( buf[n-1] == '\r' )
-			    buf[--n] = 0;
-			if( opt.verbose ) {
-			    log_info(_("armor header: "));
-			    print_string( stderr, buf, n, 0 );
-			    putc('\n', stderr);
-			}
-			if( clearsig && !(hashes=parse_hash_header( buf )) ) {
-			    if( strlen(buf) > 15
-				&& !memcmp( buf, "NotDashEscaped:", 15 ) ) {
-				*not_dashed = 1;
-				state = fhdrWAITHeader;
-			    }
-			    else {
-				log_error(_("invalid clearsig header\n"));
-				state = fhdrERROR;
-			    }
-			}
-			else {
-			    state = fhdrWAITHeader;
-			    if( r_hashes )
-				*r_hashes |= hashes;
-			}
-		    }
-		    else if( clearsig && n > 15 && !memcmp(buf, "-----", 5 ) )
-			state = fhdrNullClearsig;
-		    else
-			state = fhdrCHECKDashEscaped3;
-		}
-		else if( !n || (buf[0] == '\r' && !buf[1]) ) { /* empty line */
-		    if( clearsig )
-			state = fhdrWAITClearsig;
-		    else {
-			/* this is not really correct: if we do not have
-			 * a clearsig and no armor lines we are not allowed
-			 * to have an empty line */
-			n = 0;
-			state = fhdrTEXT;
-		    }
-		}
-		else {
-		    log_error(_("invalid armor header: "));
-		    print_string( stderr, buf, n, 0 );
-		    putc('\n', stderr);
-		    state = fhdrERROR;
-		}
-	    }
-	    else if( c != -1 ) {
-		if( strchr( buf, ':') ) { /* buffer to short, but this is okay*/
-		    if( opt.verbose ) {
-			log_info(_("armor header: "));
-			print_string( stderr, buf, n, 0 );
-			fputs("[...]\n", stderr);  /* indicate it is truncated */
-		    }
-		    state = fhdrSKIPHeader;  /* skip rest of line */
-		}
-		else /* line too long */
-		    state = fhdrERROR;
-	    }
-	    else
-		state = fhdrEOF;
-	    break;
-
-	  case fhdrWAITClearsig: /* skip the empty line (for clearsig) */
-	    c = 0;
-	    for(n=0; n < buflen && (c=iobuf_get(a)) != -1 && c != '\n'; )
-		buf[n++] = c;
-	    if( c != -1 ) {
-		if( n > 15 && !memcmp(buf, "-----", 5 ) )
-		    state = fhdrNullClearsig;
-		else if( c != '\n' )
-		    state = fhdrREADClearsigNext;
-		else
-		    state = fhdrCHECKDashEscaped3;
-	    }
-	    else {
-		/* fixme: we should check whether this line continues
-		 *   it is possible that we have only read ws until here
-		 *   and more stuff is to come */
-		state = fhdrEOF;
-	    }
-	    break;
-
-	  case fhdrNullClearsig: /* zero length cleartext */
-	    state = fhdrENDClearsig;
-	    break;
-
-	  case fhdrENDClearsig:
-	  case fhdrCHECKBegin:
-	    state = state == fhdrCHECKBegin ? fhdrINITSkip : fhdrERRORShow;
-	    if( n < 15 )
-		break;	/* too short */
-	    if( memcmp( buf, "-----", 5 ) )
-		break;
-	    buf[n] = 0;
-	    p = strstr(buf+5, "-----");
-	    if( !p )
-		break;
-	    *p = 0;
-	    p += 5;
-	    if( *p == '\r' )
-		p++;
-	    if( *p )
-		break; /* garbage after dashes */
-	    p = buf+5;
-	    for(i=0; (s=head_strings[i]); i++ )
-		if( !strcmp(s, p) )
-		    break;
-	    if( !s )
-		break; /* unknown begin line */
-	    if( only_keyblocks && i != 1 && i != 5 && i != 6 )
-		break; /* not a keyblock armor */
-
-	    /* found the begin line */
-	    hdr_line = i;
-	    state = fhdrWAITHeader;
-	    *not_dashed = 0;
-	    if( hdr_line == BEGIN_SIGNED_MSG_IDX )
-		clearsig = 1;
-	    if( opt.verbose > 1 )
-		log_info(_("armor: %s\n"), head_strings[hdr_line]);
-	    break;
-
-	  case fhdrCLEARSIGSimple:
-	    /* we are at the begin of a new line */
-	  case fhdrCLEARSIGSimpleNext:
-	    n = 0;
-	    c = 0;
-	    while( n < buflen && (c=iobuf_get(a)) != -1 ) {
-		buf[n++] = c;
-		if( c == '\n' )
-		    break;
-	    }
-	    buf[n] = 0;
-	    if( c == -1 )
-		state = fhdrEOF;
-	    else if( state == fhdrCLEARSIGSimple
-		     && n > 15 && !memcmp(buf, "-----", 5 ) ) {
-		if( c == '\n' )
-		    buf[n-1] = 0;
-		state = fhdrENDClearsig;
-	    }
-	    else if( c == '\n' )
-		state = fhdrCLEARSIGSimple;
-	    else
-		state = fhdrCLEARSIGSimpleNext;
-	    break;
-
-	  case fhdrCLEARSIG:
-	  case fhdrEMPTYClearsig:
-	  case fhdrREADClearsig:
-	    /* we are at the start of a line: read a clearsig into the buffer
-	     * we have to look for a header line or dashed escaped text*/
-	    n = 0;
-	    c = 0;
-	    while( n < buflen && (c=iobuf_get(a)) != -1 && c != '\n' )
-		buf[n++] = c;
-	    buf[n] = 0;
-	    if( c == -1 )
-		state = fhdrEOF;
-	    else if( !n || ( buf[0]=='\r' && !buf[1] ) ) {
-		state = fhdrEMPTYClearsig;
-		empty++;
-	    }
-	    else if( c == '\n' )
-		state = fhdrCHECKClearsig2;
-	    else
-		state = fhdrCHECKClearsig;
-	    break;
-
-	  case fhdrCHECKDashEscaped3:
-	    if( *not_dashed ) {
-		state = fhdrTEXTSimple;
-		break;
-	    }
-	    if( !(n > 1 && buf[0] == '-' && buf[1] == ' ' ) ) {
-		state = fhdrTEXT;
-		break;
-	    }
-	    /* fall through */
-	  case fhdrCHECKDashEscaped2:
-	  case fhdrCHECKDashEscaped:
-	    /* check dash escaped line */
-	    if( buf[2] == '-' || ( n > 6 && !memcmp(buf+2, "From ", 5))) {
-		for(i=2; i < n; i++ )
-		    buf[i-2] = buf[i];
-		n -= 2;
-		buf[n] = 0; /* not really needed */
-		state = state == fhdrCHECKDashEscaped3 ? fhdrTEXT :
-			state == fhdrCHECKDashEscaped2 ?
-				 fhdrREADClearsig : fhdrTESTSpaces;
-	    }
-	    else {
-		log_error(_("invalid dash escaped line: "));
-		print_string( stderr, buf, n, 0 );
-		putc('\n', stderr);
-		state = fhdrERROR;
-	    }
-	    break;
-
-	  case fhdrCHECKClearsig:
-	    /* check the clearsig line */
-	    if( n > 15 && !memcmp(buf, "-----", 5 ) )
-		state = fhdrENDClearsig;
-	    else if( buf[0] == '-' && buf[1] == ' ' && !*not_dashed )
-		state = fhdrCHECKDashEscaped;
-	    else {
-		state = fhdrTESTSpaces;
-	    }
-	    break;
-
-	  case fhdrCHECKClearsig2:
-	    /* check the clearsig line */
-	    if( n > 15 && !memcmp(buf, "-----", 5 ) )
-		state = fhdrENDClearsig;
-	    else if( buf[0] == '-' && buf[1] == ' ' && !*not_dashed )
-		state = fhdrCHECKDashEscaped2;
-	    else {
-		state = fhdrREADClearsig;
-	    }
-	    break;
-
-	  case fhdrREADClearsigNext:
-	    /* Read to the end of the line, do not care about checking
-	     * for dashed escaped text of headers */
-	    c = 0;
-	    n = 0;
-	    while( n < buflen && (c=iobuf_get(a)) != -1 && c != '\n' )
-		buf[n++] = c;
-	    buf[n] = 0;
-	    if( c == -1 )
-		state = fhdrEOF;
-	    else if( c == '\n' )
-		state = fhdrREADClearsig;
-	    else
-		state = fhdrTESTSpaces;
-	    break;
-
-	  case fhdrTESTSpaces: {
-	    /* but must check whether the rest of the line
-	     * only contains white spaces; this is problematic
-	     * since we may have to restore the stuff.	simply
-	     * counting spaces is not enough, because it may be a
-	     * mix of different white space characters */
-	    IOBUF b = iobuf_temp();
-	    while( (c=iobuf_get(a)) != -1 && c != '\n' ) {
-		iobuf_put(b,c);
-		if( c != ' ' && c != '\t' && c != '\r' )
-		    break;
-	    }
-	    if( c == '\n' ) {
-		/* okay we can skip the rest of the line */
-		iobuf_close(b);
-		state = fhdrREADClearsig;
-	    }
-	    else {
-		iobuf_unget_and_close_temp(a,b);
-		state = fhdrREADClearsigNext;
-	    }
-	  } break;
-
-	  case fhdrERRORShow:
-	    log_error(_("invalid clear text header: "));
-	    print_string( stderr, buf, n, 0 );
-	    putc('\n', stderr);
-	    state = fhdrERROR;
-	    break;
-
-	  default: BUG();
+    for(mark=NULL, p=line, n=0; n < len; n++, p++ ) {
+	if( strchr(" \t\r\n", *p ) ) {
+	    if( !mark )
+		mark = p;
 	}
-	switch( state ) {
-	  case fhdrINIT:
-	  case fhdrINITCont:
-	  case fhdrINITSkip:
-	  case fhdrCHECKBegin:
-	  case fhdrWAITHeader:
-	  case fhdrWAITClearsig:
-	  case fhdrSKIPHeader:
-	  case fhdrEMPTYClearsig:
-	  case fhdrCHECKClearsig:
-	  case fhdrCHECKClearsig2:
-	  case fhdrCHECKDashEscaped:
-	  case fhdrCHECKDashEscaped2:
-	  case fhdrCHECKDashEscaped3:
-	  case fhdrTESTSpaces:
-	  case fhdrERRORShow:
-	    cont = 1;
-	    break;
-	  default: cont = 0;
-	}
-    } while( cont );
-
-    if( clearsig && state == fhdrTEXT )
-	state = fhdrCLEARSIG;
-    else if( clearsig && state == fhdrTEXTSimple ) {
-	state = fhdrCLEARSIGSimple;
-	buf[n] = '\n';
-	n++;
+	else
+	    mark = NULL;
     }
 
-    if( state == fhdrCLEARSIG || state == fhdrREADClearsig ) {
-	/* append CR,LF after removing trailing wspaces */
-	for(p=buf+n-1; n; n--, p-- ) {
-	    assert( *p != '\n' );
-	    if( *p != ' ' && *p != '\t' && *p != '\r' ) {
-		p[1] = '\r';
-		p[2] = '\n';
-		n += 2;
-		break;
-	    }
-	}
-	if( !n ) {
-	    buf[0] = '\r';
-	    buf[1] = '\n';
-	    n = 2;
-	}
+    if( mark ) {
+	*mark = 0;
+	return mark - line;
     }
-
-    *r_buflen = n;
-    *r_empty = empty;
-    return state;
+    return len;
 }
 
 
+
+/****************
+ * Check whether this is a armor line.
+ * returns: -1 if it is not a armor header or the index number of the
+ * armor header.
+ */
+static int
+is_armor_header( byte *line, unsigned len )
+{
+    const char *s;
+    byte *save_p, *p;
+    int save_c;
+    int i;
+
+    if( len < 15 )
+	return -1; /* too short */
+    if( memcmp( line, "-----", 5 ) )
+	return -1; /* no */
+    p = strstr( line+5, "-----");
+    if( !p )
+	return -1;
+    save_p = p;
+    p += 5;
+    if( *p == '\r' )
+	p++;
+    if( *p == '\n' )
+	p++;
+    if( *p )
+	return -1; /* garbage after dashes */
+    save_c = *save_p; *save_p = 0;
+    p = line+5;
+    for(i=0; (s=head_strings[i]); i++ )
+	if( !strcmp(s, p) )
+	    break;
+    *save_p = save_c;
+    if( !s )
+	return -1; /* unknown armor line */
+
+    if( opt.verbose > 1 )
+	log_info(_("armor: %s\n"), head_strings[i]);
+    return i;
+}
+
+
+
+/****************
+ * Parse a header lines
+ * Return 0: Empty line (end of header lines)
+ *	 -1: invalid header line
+ *	 >0: Good header line
+ */
+static int
+parse_header_line( armor_filter_context_t *afx, byte *line, unsigned len )
+{
+    byte *p;
+    int hashes=0;
+
+    if( *line == '\n' || ( len && (*line == '\r' && line[1]=='\n') ) )
+	return 0; /* empty line */
+    len = trim_trailing_spaces( line, len );
+    p = strchr( line, ':');
+    if( !p || !p[1] ) {
+	log_error(_("invalid armor header: "));
+	print_string( stderr, line, len, 0 );
+	putc('\n', stderr);
+	return -1;
+    }
+
+    if( opt.verbose ) {
+	log_info(_("armor header: "));
+	print_string( stderr, line, len, 0 );
+	putc('\n', stderr);
+    }
+
+    if( afx->in_cleartext ) {
+	if( (hashes=parse_hash_header( line )) )
+	    afx->hashes |= hashes;
+	else if( strlen(line) > 15 && !memcmp( line, "NotDashEscaped:", 15 ) )
+	    afx->not_dash_escaped = 1;
+	else {
+	    log_error(_("invalid clearsig header\n"));
+	    return -1;
+	}
+    }
+    return 1;
+}
+
+
+
 /* figure out whether the data is armored or not */
 static int
 check_input( armor_filter_context_t *afx, IOBUF a )
 {
     int rc = 0;
-    size_t n;
-    fhdr_state_t state = afx->parse_state;
-    unsigned emplines;
+    int i;
+    byte *line;
+    unsigned len;
+    unsigned maxlen;
+    int hdr_line = -1;
 
-    if( state != fhdrENDClearsig )
-	state = fhdrHASArmor;
-
-    n = DIM(afx->helpbuf);
-    state = find_header( state, afx->helpbuf, &n, a,
-				afx->helplen, &emplines, &afx->hashes,
-				afx->only_keyblocks, &afx->not_dash_escaped );
-    switch( state ) {
-      case fhdrNOArmor:
+    /* read the first line to see whether this is armored data */
+    maxlen = MAX_LINELEN;
+    len = afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+					     &afx->buffer_size, &maxlen );
+    line = afx->buffer;
+    if( !maxlen ) {
+	/* line has been truncated: assume not armored */
 	afx->inp_checked = 1;
 	afx->inp_bypass = 1;
-	afx->helplen = n;
-	break;
+	return 0;
+    }
 
-      case fhdrERROR:
+    if( !len ) {
+	return -1; /* eof */
+    }
+
+    /* (the line is always a C string but maybe longer) */
+    if( *line == '\n' || ( len && (*line == '\r' && line[1]=='\n') ) )
+	;
+    else if( !is_armored( line ) ) {
+	afx->inp_checked = 1;
+	afx->inp_bypass = 1;
+	return 0;
+    }
+
+    /* find the armor header */
+    while(len) {
+	i = is_armor_header( line, len );
+	if( i >= 0 && !(afx->only_keyblocks && i != 1 && i != 5 && i != 6 )) {
+	    hdr_line = i;
+	    if( hdr_line == BEGIN_SIGNED_MSG_IDX ) {
+		if( afx->in_cleartext ) {
+		    log_error(_("nested clear text signatures\n"));
+		    rc = G10ERR_INVALID_ARMOR;
+		}
+		afx->in_cleartext = 1;
+	    }
+	    break;
+	}
+	/* read the next line (skip all truncated lines) */
+	do {
+	    maxlen = MAX_LINELEN;
+	    afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+					       &afx->buffer_size, &maxlen );
+	    line = afx->buffer;
+	    len = afx->buffer_len;
+	} while( !maxlen );
+    }
+
+    /* parse the header lines */
+    while(len) {
+	/* read the next line (skip all truncated lines) */
+	do {
+	    maxlen = MAX_LINELEN;
+	    afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+					       &afx->buffer_size, &maxlen );
+	    line = afx->buffer;
+	    len = afx->buffer_len;
+	} while( !maxlen );
+
+	i = parse_header_line( afx, line, len );
+	if( i <= 0 ) {
+	    if( i )
+		rc = G10ERR_INVALID_ARMOR;
+	    break;
+	}
+    }
+
+
+    if( rc )
 	invalid_armor();
-	break;
-
-      case fhdrEOF:
-	rc = -1;
-	break;
-
-      case fhdrNullClearsig:
-      case fhdrCLEARSIG: /* start fake package mode (for clear signatures) */
-      case fhdrREADClearsigNext:
-      case fhdrCLEARSIGSimple:
-      case fhdrCLEARSIGSimpleNext:
-	afx->helplen = n;
-	afx->helpidx = 0;
+    else if( afx->in_cleartext ) {
 	afx->faked = 1;
-	break;
-
-      case fhdrTEXT:
-	afx->helplen = n;
-	afx->helpidx = 0;
+    }
+    else {
 	afx->inp_checked = 1;
 	afx->crc = CRCINIT;
 	afx->idx = 0;
 	afx->radbuf[0] = 0;
-	break;
-
-      default: BUG();
     }
 
-    afx->parse_state = state;
     return rc;
 }
 
 
 
-/* fake a literal data packet and wait for an armor line */
+/****************
+ * Fake a literal data packet and wait for the next armor line
+ * fixme: empty line handling and null length clear text signature are
+ *	  not implemented/checked.
+ */
 static int
 fake_packet( armor_filter_context_t *afx, IOBUF a,
 	     size_t *retn, byte *buf, size_t size  )
 {
     int rc = 0;
     size_t len = 0;
-    size_t n, nn;
-    fhdr_state_t state = afx->parse_state;
     unsigned emplines = afx->empty;
+    int lastline = 0;
+    unsigned maxlen, n;
+    byte *p;
 
     len = 2;	/* reserve 2 bytes for the length header */
     size -= 3;	/* and 1 for empline handling and 2 for the term header */
+		/* or the appended CR,LF */
     while( !rc && len < size ) {
 	if( emplines ) {
 	    while( emplines && len < size ) {
@@ -743,70 +493,100 @@ fake_packet( armor_filter_context_t *afx, IOBUF a,
 	    }
 	    continue;
 	}
-	if( state == fhdrENDClearsigHelp ) {
-	    state = fhdrENDClearsig;
-	    afx->faked = 0;
-	    rc = -1;
+
+	if( afx->faked == 1 )
+	    afx->faked++;  /* skip the first (empty) line */
+	else {
+	    while( len < size && afx->buffer_pos < afx->buffer_len )
+		buf[len++] = afx->buffer[afx->buffer_pos++];
+	    if( afx->buffer_pos >= afx->buffer_len ) {
+		buf[len++] = '\r';
+		buf[len++] = '\n';
+	    }
+	    if( len >= size )
+		continue;
+	}
+
+	/* read the next line */
+	maxlen = MAX_LINELEN;
+	afx->buffer_pos = 0;
+	afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+					   &afx->buffer_size, &maxlen );
+	if( !afx->buffer_len ) {
+	    rc = -1; /* eof */
 	    continue;
 	}
-	if( state != fhdrNullClearsig
-	    && afx->helpidx < afx->helplen ) { /* flush the last buffer */
-	    n = afx->helplen;
-	    for(nn=afx->helpidx; len < size && nn < n ; nn++ )
-		buf[len++] = afx->helpbuf[nn];
-	    afx->helpidx = nn;
-	    continue;
-	}
-	if( state == fhdrEOF ) {
-	    rc = -1;
-	    continue;
-	}
-	/* read a new one */
-	n = DIM(afx->helpbuf);
-	afx->helpidx = 0;
-	state = find_header( state, afx->helpbuf, &n, a,
-			      state == fhdrNullClearsig? afx->helplen:0,
-						&emplines, &afx->hashes,
-						afx->only_keyblocks,
-						&afx->not_dash_escaped );
-	switch( state) {
-	  case fhdrERROR:
-	    invalid_armor();
-	    break;
+	if( !maxlen )
+	    afx->truncated++;
+	afx->buffer_len = trim_trailing_spaces( afx->buffer, afx->buffer_len );
+	p = afx->buffer;
+	n = afx->buffer_len;
 
-	  case fhdrEOF:
-	    rc = -1;
-	    break;
-
-	  case fhdrCLEARSIG:
-	    BUG();
-
-	  case fhdrREADClearsig:
-	  case fhdrREADClearsigNext:
-	  case fhdrCLEARSIGSimple:
-	  case fhdrCLEARSIGSimpleNext:
-	    afx->helplen = n;
-	    break;
-
-	  case fhdrENDClearsig:
-	    state = fhdrENDClearsigHelp;
-	    afx->helplen = n;
-	    break;
-
-	  default: BUG();
+	if( n > 2 && *p == '-' ) {
+	    /* check for dash escaped or armor header */
+	    if( p[1] == ' ' && !afx->not_dash_escaped ) {
+		/* issue a warning if it is not regular encoded */
+		if( p[2] != '-' && !( n > 6 && !memcmp(p+2, "From ", 5))) {
+		    log_info(_("invalid dash escaped line: "));
+		    print_string( stderr, p, n, 0 );
+		    putc('\n', stderr);
+		}
+		afx->buffer_pos = 2; /* skip */
+	    }
+	    else if( n >= 15 &&  p[1] == '-' && p[2] == '-' && p[3] == '-' ) {
+		if( is_armor_header( p, n ) != BEGIN_SIGNATURE ) {
+		    log_info(_("unexpected armor:"));
+		    print_string( stderr, p, n, 0 );
+		    putc('\n', stderr);
+		}
+		lastline = 1;
+		if( len >= 2 )
+		    len -= 2; /* remove the last CR,LF */
+		rc = -1;
+	    }
 	}
     }
+
     buf[0] = (len-2) >> 8;
     buf[1] = (len-2);
-    if( state == fhdrENDClearsig ) { /* write last (ending) length header */
-	if( buf[0] || buf[1] ) { /* write only if length of text is > 0 */
+    if( lastline ) { /* write last (ending) length header */
+	if( buf[0] || buf[1] ) { /* only if we have some text */
 	    buf[len++] = 0;
 	    buf[len++] = 0;
 	}
 	rc = 0;
+	afx->faked = 0;
+	afx->in_cleartext = 0;
+	/* and now read the header lines */
+	afx->buffer_pos = 0;
+	for(;;) {
+	    int i;
+
+	    /* read the next line (skip all truncated lines) */
+	    do {
+		maxlen = MAX_LINELEN;
+		afx->buffer_len = iobuf_read_line( a, &afx->buffer,
+						 &afx->buffer_size, &maxlen );
+	    } while( !maxlen );
+	    p = afx->buffer;
+	    n = afx->buffer_len;
+	    if( !n ) {
+		rc = -1;
+		break; /* eof */
+	    }
+	    i = parse_header_line( afx, p , n );
+	    if( i <= 0 ) {
+		if( i )
+		    invalid_armor();
+		break;
+	    }
+	}
+	afx->inp_checked = 1;
+	afx->crc = CRCINIT;
+	afx->idx = 0;
+	afx->radbuf[0] = 0;
     }
 
-    afx->parse_state = state;
     afx->empty = emplines;
     *retn = len;
     return rc;
@@ -830,9 +610,7 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
     idx = afx->idx;
     val = afx->radbuf[0];
     for( n=0; n < size; ) {
-	if( afx->helpidx < afx->helplen )
-	    c = afx->helpbuf[afx->helpidx++];
-	else if( (c=iobuf_get(a)) == -1 )
+	if( (c=iobuf_get(a)) == -1 )
 	    break;
 	if( c == '\n' || c == ' ' || c == '\r' || c == '\t' )
 	    continue;
@@ -864,11 +642,8 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 	afx->any_data = 1;
 	afx->inp_checked=0;
 	afx->faked = 0;
-	afx->parse_state = 0;
 	for(;;) { /* skip lf and pad characters */
-	    if( afx->helpidx < afx->helplen )
-		c = afx->helpbuf[afx->helpidx++];
-	    else if( (c=iobuf_get(a)) == -1 )
+	    if( (c=iobuf_get(a)) == -1 )
 		break;
 	    if( c == '\n' || c == ' ' || c == '\r'
 		|| c == '\t' || c == '=' )
@@ -889,9 +664,7 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		  case 2: val |= (c>>2)&15; mycrc |= val << 8;val=(c<<6)&0xc0;break;
 		  case 3: val |= c&0x3f; mycrc |= val; break;
 		}
-		if( afx->helpidx < afx->helplen )
-		    c = afx->helpbuf[afx->helpidx++];
-		else if( (c=iobuf_get(a)) == -1 )
+		if( (c=iobuf_get(a)) == -1 )
 		    break;
 	    } while( ++idx < 4 );
 	    if( c == -1 ) {
@@ -913,9 +686,7 @@ radix64_read( armor_filter_context_t *afx, IOBUF a, size_t *retn,
 		for(rc=0;!rc;) {
 		    rc = 0 /*check_trailer( &fhdr, c )*/;
 		    if( !rc ) {
-			if( afx->helpidx < afx->helplen )
-			    c = afx->helpbuf[afx->helpidx++];
-			else if( (c=iobuf_get(a)) == -1 )
+			if( (c=iobuf_get(a)) == -1 )
 			    rc = 2;
 		    }
 		}
@@ -967,7 +738,14 @@ armor_filter( void *opaque, int control,
     if( DBG_FILTER )
 	log_debug("armor-filter: control: %d\n", control );
     if( control == IOBUFCTRL_UNDERFLOW && afx->inp_bypass ) {
-	for( n=0; n < size; n++ ) {
+	n = 0;
+	if( afx->buffer_len ) {
+	    for(; n < size && afx->buffer_pos < afx->buffer_len; n++ )
+		buf[n++] = afx->buffer[afx->buffer_pos++];
+	    if( afx->buffer_pos >= afx->buffer_len )
+		afx->buffer_len = 0;
+	}
+	for(; n < size; n++ ) {
 	    if( (c=iobuf_get(a)) == -1 )
 		break;
 	    buf[n] = c & 0xff;
@@ -985,12 +763,12 @@ armor_filter( void *opaque, int control,
 	else if( !afx->inp_checked ) {
 	    rc = check_input( afx, a );
 	    if( afx->inp_bypass ) {
-		for( n=0; n < size && n < afx->helplen; n++ )
-		    buf[n] = afx->helpbuf[n];
+		for(n=0; n < size && afx->buffer_pos < afx->buffer_len; n++ )
+		    buf[n++] = afx->buffer[afx->buffer_pos++];
+		if( afx->buffer_pos >= afx->buffer_len )
+		    afx->buffer_len = 0;
 		if( !n )
 		    rc = -1;
-		assert( n == afx->helplen );
-		afx->helplen = 0;
 	    }
 	    else if( afx->faked ) {
 		unsigned hashes = afx->hashes;
@@ -1080,7 +858,7 @@ armor_filter( void *opaque, int control,
 	    }
 	    else
 		iobuf_writestr(a,
-		    "Comment: For info see www.gnupg.org");
+		    "Comment: For info see http://www.gnupg.org\n");
 	    if( afx->hdrlines )
 		iobuf_writestr(a, afx->hdrlines);
 	    iobuf_put(a, '\n');
@@ -1181,6 +959,11 @@ armor_filter( void *opaque, int control,
 	}
 	else if( !afx->any_data && !afx->inp_bypass )
 	    log_error(_("no valid OpenPGP data found.\n"));
+	if( afx->truncated )
+	    log_info(_("invalid armor: line longer than %d characters\n"),
+		      MAX_LINELEN );
+	m_free( afx->buffer );
+	afx->buffer = NULL;
     }
     else if( control == IOBUFCTRL_DESC )
 	*(char**)buf = "armor_filter";
