@@ -264,7 +264,6 @@ free_encrypted( PKT_encrypted *ed )
 	if( iobuf_in_block_mode(ed->buf) ) {
 	    while( iobuf_get(ed->buf) != -1 )
 		;
-	    iobuf_set_block_mode(ed->buf, 0);
 	}
 	else {
 	    for( ; ed->len; ed->len-- ) /* skip the packet */
@@ -282,7 +281,6 @@ free_plaintext( PKT_plaintext *pt )
 	if( iobuf_in_block_mode(pt->buf) ) {
 	    while( iobuf_get(pt->buf) != -1 )
 		;
-	    iobuf_set_block_mode(pt->buf, 0);
 	}
 	else {
 	    for( ; pt->len; pt->len-- ) /* skip the packet */
@@ -354,7 +352,7 @@ cmp_public_keys( PKT_public_key *a, PKT_public_key *b )
 
     if( a->timestamp != b->timestamp )
 	return -1;
-    if( a->expiredate != b->expiredate )
+    if( a->version < 4 && a->expiredate != b->expiredate )
 	return -1;
     if( a->pubkey_algo != b->pubkey_algo )
 	return -1;
@@ -381,7 +379,7 @@ cmp_secret_keys( PKT_secret_key *a, PKT_secret_key *b )
 
     if( a->timestamp != b->timestamp )
 	return -1;
-    if( a->expiredate != b->expiredate )
+    if( a->version < 4 && a->expiredate != b->expiredate )
 	return -1;
     if( a->pubkey_algo != b->pubkey_algo )
 	return -1;
@@ -407,7 +405,7 @@ cmp_public_secret_key( PKT_public_key *pk, PKT_secret_key *sk )
 
     if( pk->timestamp != sk->timestamp )
 	return -1;
-    if( pk->expiredate != sk->expiredate )
+    if( pk->version < 4 && pk->expiredate != sk->expiredate )
 	return -1;
     if( pk->pubkey_algo != sk->pubkey_algo )
 	return -1;
