@@ -195,7 +195,7 @@ proc_pubkey_enc( CTX c, PACKET *pkt )
 	;
     else if( !result ) {
 	if( opt.verbose > 1 )
-	    log_info( _("public key encrypted data: Good DEK\n") );
+	    log_info( _("public key encrypted data: good DEK\n") );
     }
     else {
 	log_error(_("public key decryption failed: %s\n"), g10_errstr(result));
@@ -849,6 +849,24 @@ check_sig_and_print( CTX c, KBNODE node )
 	print_keyid( stderr, sig->keyid );
 	putc('\"', stderr);
 	putc('\n', stderr);
+	if( !rc && is_status_enabled() ) {
+	    /* print a status response with the fingerprint */
+	    PKT_public_key *pk = m_alloc_clear( sizeof *pk );
+
+	    if( !get_pubkey( pk, sig->keyid ) ) {
+		byte array[MAX_FINGERPRINT_LEN], *p;
+		char buf[MAX_FINGERPRINT_LEN*2+1];
+		size_t i, n;
+
+		fingerprint_from_pk( pk, array, &n );
+		p = array;
+		for(i=0; i < n ; i++, p++ )
+		    sprintf(buf+2*i, "%02X", *p );
+		write_status_text( STATUS_VALIDSIG, buf );
+	    }
+	    free_public_key( pk );
+	}
+
 	if( !rc )
 	    rc = check_signatures_trust( sig );
 	if( rc )
