@@ -922,7 +922,7 @@ parse_sig_subpkt( const byte *buffer, sigsubpkttype_t reqtype, size_t *ret_n )
 	buffer += n; buflen -=n;
     }
     if( reqtype == SIGSUBPKT_TEST_CRITICAL )
-	return buffer; /* as value true to indicate that there is no
+	return buffer; /* as value true to indicate that there is no */
 		       /* critical bit we don't understand */
     return NULL; /* end of packets; not found */
 
@@ -1201,6 +1201,7 @@ parse_key( IOBUF inp, int pkttype, unsigned long pktlen,
 	sk->expiredate = expiredate;
 	sk->hdrbytes = hdrlen;
 	sk->version = version;
+	sk->is_primary = pkttype == PKT_SECRET_KEY;
 	sk->pubkey_algo = algorithm;
 	sk->pubkey_usage = 0; /* not yet used */
     }
