@@ -264,7 +264,7 @@ _("Could not find a valid trust path to the key.  Let's see whether we\n"
     else if( !changed )
 	tty_printf(_("No trust values changed.\n\n") );
 
-    return any? 0:-1;
+    return changed? 0:-1;
 }
 
 /****************
@@ -369,7 +369,9 @@ do_we_trust_pre( PKT_public_key *pk, int trustlevel )
 {
     int rc = do_we_trust( pk, trustlevel );
 
-    if( !opt.batch && !rc ) {
+    if( (trustlevel & TRUST_FLAG_REVOKED) && !rc )
+	return 0;
+    else if( !opt.batch && !rc ) {
 	tty_printf(_(
 "It is NOT certain that the key belongs to its owner.\n"
 "If you *really* know what you are doing, you may answer\n"
@@ -541,7 +543,7 @@ build_pk_list( STRLIST remusr, PK_LIST *ret_pk_list, unsigned usage )
 
 		rc = check_trust( pk, &trustlevel );
 		if( rc ) {
-		    log_error("error checking pk of '%s': %s\n",
+		    log_error("error checking pk of `%s': %s\n",
 						      answer, g10_errstr(rc) );
 		}
 		else if( do_we_trust_pre( pk, trustlevel ) ) {
