@@ -1,5 +1,5 @@
 /* plaintext.c -  process an plaintext packet
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <errno.h>
+#include <assert.h>
 #include "util.h"
 #include "memory.h"
 #include "options.h"
@@ -34,46 +35,6 @@
 #include "i18n.h"
 
 
-/****************
- * Defer the last CR,LF
- */
-static void
-special_md_putc( MD_HANDLE md, int c, int *state )
-{
-    if( c == -1 ) { /* flush */
-	if( *state == 1 ) {
-	    md_putc(md, '\r');
-	}
-
-	*state = 0;
-	return;
-    }
-  again:
-    switch( *state ) {
-      case 0:
-	if( c == '\r' )
-	    *state = 1;
-	else
-	    md_putc(md, c );
-	break;
-      case 1:
-	if( c == '\n' )
-	    *state = 2;
-	else {
-	    md_putc(md, '\r');
-	    *state = 0;
-	    goto again;
-	}
-	break;
-      case 2:
-	md_putc(md, '\r');
-	md_putc(md, '\n');
-	*state = 0;
-	goto again;
-      default: BUG();
-    }
-}
-
 
 /****************
  * Handle a plaintext packet.  If MFX is not NULL, update the MDs
@@ -90,7 +51,6 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     int rc = 0;
     int c;
     int convert = pt->mode == 't';
-    int special_state = 0;
 
     /* create the filename as C string */
     if( nooutput )
@@ -123,7 +83,7 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     if( fp || nooutput )
 	;
     else if( !(fp = fopen(fname,"wb")) ) {
-	log_error("Error creating '%s': %s\n", fname, strerror(errno) );
+	log_error("Error creating `%s': %s\n", fname, strerror(errno) );
 	rc = G10ERR_CREATE_FILE;
 	goto leave;
     }
@@ -136,17 +96,13 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
 		rc = G10ERR_READ_FILE;
 		goto leave;
 	    }
-	    if( mfx->md ) {
-		if( convert && clearsig )
-		    special_md_putc(mfx->md, c, &special_state );
-		else
-		    md_putc(mfx->md, c );
-	    }
+	    if( mfx->md )
+		md_putc(mfx->md, c );
 	    if( convert && !clearsig && c == '\r' )
 		continue; /* fixme: this hack might be too simple */
 	    if( fp ) {
 		if( putc( c, fp ) == EOF ) {
-		    log_error("Error writing to '%s': %s\n",
+		    log_error("Error writing to `%s': %s\n",
 					    fname, strerror(errno) );
 		    rc = G10ERR_WRITE_FILE;
 		    goto leave;
@@ -156,30 +112,24 @@ handle_plaintext( PKT_plaintext *pt, md_filter_context_t *mfx,
     }
     else {
 	while( (c = iobuf_get(pt->buf)) != -1 ) {
-	    if( mfx->md ) {
-		if( convert && clearsig )
-		    special_md_putc(mfx->md, c, &special_state	);
-		else
-		    md_putc(mfx->md, c );
-	    }
+	    if( mfx->md )
+		md_putc(mfx->md, c );
 	    if( convert && !clearsig && c == '\r' )
 		continue; /* fixme: this hack might be too simple */
 	    if( fp ) {
 		if( putc( c, fp ) == EOF ) {
-		    log_error("Error writing to '%s': %s\n",
+		    log_error("Error writing to `%s': %s\n",
 						fname, strerror(errno) );
 		    rc = G10ERR_WRITE_FILE;
 		    goto leave;
 		}
 	    }
 	}
-	iobuf_clear_eof(pt->buf);
+	pt->buf = NULL;
     }
-    if( mfx->md && convert && clearsig )
-	special_md_putc(mfx->md, -1, &special_state  ); /* flush */
 
     if( fp && fp != stdout && fclose(fp) ) {
-	log_error("Error closing '%s': %s\n", fname, strerror(errno) );
+	log_error("Error closing `%s': %s\n", fname, strerror(errno) );
 	fp = NULL;
 	rc = G10ERR_WRITE_FILE;
 	goto leave;
@@ -225,7 +175,7 @@ ask_for_detached_datafile( md_filter_context_t *mfx, const char *inname )
 		any++;
 	    }
 	    else if( !fp ) {
-		log_error("can't open '%s': %s\n", answer, strerror(errno) );
+		log_error("can't open `%s': %s\n", answer, strerror(errno) );
 		rc = G10ERR_READ_FILE;
 		goto leave;
 	    }
@@ -297,7 +247,7 @@ hash_datafiles( MD_HANDLE md, STRLIST files,
     for( ; sl; sl = sl->next ) {
 	fp = iobuf_open( sl->d );
 	if( !fp ) {
-	    log_error(_("can't open signed data '%s'\n"),
+	    log_error(_("can't open signed data `%s'\n"),
 						print_fname_stdin(sl->d));
 	    if( !files )
 		free_strlist(sl);
