@@ -1,5 +1,5 @@
 /* iobuf.c  -  file handling
- *	Copyright (C) 1998 Free Software Foundation, Inc.
+ *	Copyright (C) 1998, 1999 Free Software Foundation, Inc.
  *
  * This file is part of GnuPG.
  *
@@ -401,6 +401,24 @@ block_filter(void *opaque, int control, IOBUF chain, byte *buf, size_t *ret_len)
 }
 
 
+static void
+print_chain( IOBUF a )
+{
+    if( !DBG_IOBUF )
+	return;
+    for(; a; a = a->chain ) {
+	size_t dummy_len = 0;
+	const char *desc = "[none]";
+
+	if( a->filter )
+	    a->filter( a->filter_ov, IOBUFCTRL_DESC, NULL,
+						(byte*)&desc, &dummy_len );
+
+	log_debug("iobuf chain: %d.%d `%s' filter_eof=%d start=%d len=%d\n",
+		   a->no, a->subno, desc, a->filter_eof,
+		   a->d.start, a->d.len );
+    }
+}
 
 /****************
  * Allocate a new io buffer, with no function assigned.
@@ -434,7 +452,7 @@ iobuf_close( IOBUF a )
     if( a && a->directfp ) {
 	fclose( a->directfp );
 	if( DBG_IOBUF )
-	    log_debug("iobuf-close -> %p\n", a->directfp );
+	    log_debug("iobuf_close -> %p\n", a->directfp );
 	return 0;
     }
 
@@ -444,7 +462,7 @@ iobuf_close( IOBUF a )
 	    log_error("iobuf_flush failed on close: %s\n", g10_errstr(rc));
 
 	if( DBG_IOBUF )
-	    log_debug("iobuf-%d.%d: close '%s'\n", a->no, a->subno, a->desc );
+	    log_debug("iobuf-%d.%d: close `%s'\n", a->no, a->subno, a->desc );
 	if( a->filter && (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE,
 					 a->chain, NULL, &dummy_len)) )
 	    log_error("IOBUFCTRL_FREE failed on close: %s\n", g10_errstr(rc) );
@@ -526,7 +544,7 @@ iobuf_open( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: open '%s'\n", a->no, a->subno, fname );
+	log_debug("iobuf-%d.%d: open `%s'\n", a->no, a->subno, fname );
 
     return a;
 }
@@ -560,7 +578,7 @@ iobuf_create( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: create '%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: create `%s'\n", a->no, a->subno, a->desc );
 
     return a;
 }
@@ -590,7 +608,7 @@ iobuf_append( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: append '%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: append `%s'\n", a->no, a->subno, a->desc );
 
     return a;
 }
@@ -616,7 +634,7 @@ iobuf_openrw( const char *fname )
     file_filter( fcx, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &len );
     file_filter( fcx, IOBUFCTRL_INIT, NULL, NULL, &len );
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: openrw '%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: openrw `%s'\n", a->no, a->subno, a->desc );
 
     return a;
 }
@@ -708,9 +726,8 @@ iobuf_push_filter( IOBUF a,
     f( ov, IOBUFCTRL_DESC, NULL, (byte*)&a->desc, &dummy_len );
 
     if( DBG_IOBUF ) {
-	log_debug("iobuf-%d.%d: push '%s'\n", a->no, a->subno, a->desc );
-	for(b=a; b; b = b->chain )
-	    log_debug("\tchain: %d.%d '%s'\n", b->no, b->subno, b->desc );
+	log_debug("iobuf-%d.%d: push `%s'\n", a->no, a->subno, a->desc );
+	print_chain( a );
     }
 
     /* now we can initialize the new function if we have one */
@@ -722,9 +739,10 @@ iobuf_push_filter( IOBUF a,
 
 /****************
  * Remove an i/o filter.
+ * Only needed for iobuf_seek?
  */
-int
-iobuf_pop_filter( IOBUF a, int (*f)(void *opaque, int control,
+static int
+pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 		      IOBUF chain, byte *buf, size_t *len), void *ov )
 {
     IOBUF b;
@@ -735,7 +753,7 @@ iobuf_pop_filter( IOBUF a, int (*f)(void *opaque, int control,
 	BUG();
 
     if( DBG_IOBUF )
-	log_debug("iobuf-%d.%d: pop '%s'\n", a->no, a->subno, a->desc );
+	log_debug("iobuf-%d.%d: pop `%s'\n", a->no, a->subno, a->desc );
     if( !a->filter ) { /* this is simple */
 	b = a->chain;
 	assert(b);
@@ -798,16 +816,27 @@ underflow(IOBUF a)
     size_t len;
     int rc;
 
-  /*log_debug("iobuf-%d.%d: underflow: start=%lu len=%lu\n",
-		a->no, a->subno, (ulong)a->d.start, (ulong)a->d.len );*/
     assert( a->d.start == a->d.len );
     if( a->usage == 3 )
 	return -1; /* EOF because a temp buffer can't do an underflow */
 
     if( a->filter_eof ) {
+	if( a->chain && a->filter_eof == 1 ) {
+	    IOBUF b = a->chain;
+	    if( DBG_IOBUF )
+		log_debug("iobuf-%d.%d: pop `%s' in underflow\n",
+					a->no, a->subno, a->desc );
+	    m_free(a->d.buf);
+	    memcpy(a, b, sizeof *a);
+	    m_free(b);
+	    print_chain(a);
+	}
+	else
+	    a->filter_eof = 0;
 	if( DBG_IOBUF )
-	    log_debug("iobuf-%d.%d: filter eof\n", a->no, a->subno );
-	return -1;
+	    log_debug("iobuf-%d.%d: underflow: eof (due to filter eof)\n",
+						    a->no, a->subno );
+	return -1; /* return one(!) EOF */
     }
     if( a->error ) {
 	if( DBG_IOBUF )
@@ -822,8 +851,6 @@ underflow(IOBUF a)
 	if( len < a->d.size ) {
 	    if( ferror(fp) )
 		a->error = 1;
-	    else if( feof( fp ) )
-		a->filter_eof = 1;
 	}
 	a->d.len = len;
 	a->d.start = 0;
@@ -835,47 +862,60 @@ underflow(IOBUF a)
 	len = a->d.size;
 	rc = a->filter( a->filter_ov, IOBUFCTRL_UNDERFLOW, a->chain,
 			a->d.buf, &len );
+	if( DBG_IOBUF ) {
+	    log_debug("iobuf-%d.%d: underflow: req=%lu got=%lu rc=%d\n",
+		    a->no, a->subno, (ulong)a->d.size, (ulong)len, rc );
+	  #if 0
+	    if( a->no == 7 ) {
+		print_string(stderr, a->d.buf, len, 0 );
+		putc('\n', stderr );
+	    }
+	  #endif
+
+	}
 	if( a->usage == 1 && rc == -1 ) { /* EOF: we can remove the filter */
 	    size_t dummy_len;
 
-	    /* and tell the filter to free it self */
-	    if( a->filter != file_filter ) {
-		if( (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE, a->chain,
-				   NULL, &dummy_len)) )
-		    log_error("IOBUFCTRL_FREE failed: %s\n", g10_errstr(rc) );
-		a->filter = NULL;
-		a->desc = NULL;
-		a->filter_ov = NULL;
-	    }
+	    /* and tell the filter to free itself */
+	    if( (rc = a->filter(a->filter_ov, IOBUFCTRL_FREE, a->chain,
+			       NULL, &dummy_len)) )
+		log_error("IOBUFCTRL_FREE failed: %s\n", g10_errstr(rc) );
+	    a->filter = NULL;
+	    a->desc = NULL;
+	    a->filter_ov = NULL;
 	    a->filter_eof = 1;
+	    if( !len && a->chain ) {
+		IOBUF b = a->chain;
+		if( DBG_IOBUF )
+		    log_debug("iobuf-%d.%d: pop `%s' in underflow (!len)\n",
+					       a->no, a->subno, a->desc );
+		print_chain(a);
+		m_free(a->d.buf);
+		memcpy(a,b, sizeof *a);
+		m_free(b);
+		print_chain(a);
+
+
+	    }
 	}
 	else if( rc )
 	    a->error = 1;
 
-	if( !len )
+	if( !len ) {
+	    if( DBG_IOBUF )
+		log_debug("iobuf-%d.%d: underflow: eof\n", a->no, a->subno );
 	    return -1;
+	}
 	a->d.len = len;
 	a->d.start = 0;
 	return a->d.buf[a->d.start++];
     }
-    else
+    else {
+	if( DBG_IOBUF )
+	    log_debug("iobuf-%d.%d: underflow: eof (no filter)\n",
+						    a->no, a->subno );
 	return -1;  /* no filter; return EOF */
-}
-
-
-void
-iobuf_clear_eof(IOBUF a)
-{
-    if( a->directfp )
-	return;
-
-    assert(a->usage == 1);
-
-    if( a->filter )
-	log_info("iobuf-%d.%d: clear_eof '%s' with enabled filter\n", a->no, a->subno, a->desc );
-    if( !a->filter_eof )
-	log_info("iobuf-%d.%d: clear_eof '%s' with no EOF pending\n", a->no, a->subno, a->desc );
-    iobuf_pop_filter(a, NULL, NULL);
+    }
 }
 
 
@@ -999,6 +1039,7 @@ iobuf_peek(IOBUF a, byte *buf, unsigned buflen )
 
     if( a->filter_eof )
 	return -1;
+
     if( !(a->d.start < a->d.len) ) {
 	if( underflow(a) == -1 )
 	    return -1;
@@ -1104,6 +1145,7 @@ iobuf_unget_and_close_temp( IOBUF a, IOBUF temp )
     a->unget.buf = m_alloc( a->unget.size );
     a->nofast |= 2;
     a->unget.len = temp->d.len;
+    a->unget.start = 0;
     memcpy( a->unget.buf, temp->d.buf, a->unget.len );
     iobuf_close(temp);
 }
@@ -1208,8 +1250,10 @@ iobuf_seek( IOBUF a, ulong newpos )
     a->ntotal = newpos;
     a->error = 0;
     /* remove filters, but the last */
+    if( a->chain )
+	log_debug("pop_filter called in iobuf_seek - please report\n");
     while( a->chain )
-	iobuf_pop_filter( a, a->filter, NULL );
+	pop_filter( a, a->filter, NULL );
 
     return 0;
 }
@@ -1262,7 +1306,9 @@ iobuf_set_block_mode( IOBUF a, size_t n )
     assert( a->usage == 1 || a->usage == 2 );
     ctx->usage = a->usage;
     if( !n ) {
-	iobuf_pop_filter(a, block_filter, NULL );
+	if( a->usage == 1 )
+	    log_debug("pop_filter called in set_block_mode - please report\n");
+	pop_filter(a, block_filter, NULL );
     }
     else {
 	ctx->size = n; /* only needed for usage 2 */
@@ -1282,7 +1328,10 @@ iobuf_set_partial_block_mode( IOBUF a, size_t len )
     assert( a->usage == 1 || a->usage == 2 );
     ctx->usage = a->usage;
     if( !len ) {
-	iobuf_pop_filter(a, block_filter, NULL );
+	if( a->usage == 1 )
+	    log_debug("pop_filter called in set_partial_block_mode"
+						    " - please report\n");
+	pop_filter(a, block_filter, NULL );
     }
     else {
 	ctx->partial = 1;
@@ -1306,4 +1355,62 @@ iobuf_in_block_mode( IOBUF a )
 }
 
 
+/****************
+ * Same as fgets() but if the buffer is too short a larger one will
+ * be allocated up to some limit *max_length.
+ * A line is considered a byte stream ending in a LF.
+ * Returns the length of the line. EOF is indicated by a line of
+ * length zero. The last LF may be missing due to an EOF.
+ * is max_length is zero on return, the line has been truncated.
+ *
+ * Note: The buffer is allocated with enough space to append a CR,LF,EOL
+ */
+unsigned
+iobuf_read_line( IOBUF a, byte **addr_of_buffer,
+			  unsigned *length_of_buffer, unsigned *max_length )
+{
+    int c;
+    char *buffer = *addr_of_buffer;
+    unsigned length = *length_of_buffer;
+    unsigned nbytes = 0;
+    unsigned maxlen = *max_length;
+    char *p;
+
+    if( !buffer ) { /* must allocate a new buffer */
+	length = 256;
+	buffer = m_alloc( length );
+	*addr_of_buffer = buffer;
+	*length_of_buffer = length;
+    }
+
+    length -= 3; /* reserve 3 bytes (cr,lf,eol) */
+    p = buffer;
+    while( (c=iobuf_get(a)) != -1 ) {
+	if( nbytes == length ) { /* increase the buffer */
+	    if( length > maxlen  ) { /* this is out limit */
+		/* skip the rest of the line */
+		while( c != '\n' && (c=iobuf_get(a)) != -1 )
+		    ;
+		*p++ = '\n'; /* always append a LF (we have reserved space) */
+		nbytes++;
+		*max_length = 0; /* indicate truncation */
+		break;
+	    }
+	    length += 3; /* correct for the reserved byte */
+	    length += length < 1024? 256 : 1024;
+	    buffer = m_realloc( buffer, length );
+	    *addr_of_buffer = buffer;
+	    *length_of_buffer = length;
+	    length -= 3; /* and reserve again */
+	    p = buffer + nbytes;
+	}
+	*p++ = c;
+	nbytes++;
+	if( c == '\n' )
+	    break;
+    }
+    *p = 0; /* make sure the line is a string */
+
+    return nbytes;
+}
 
